package PureMessage::Install::Custom;
use strict;
use PureMessage::Install;
use PureMessage::Install::Log;
use PureMessage::Install::Utils qw(get_iq_conf get_install_defaults is_installed);
our @ISA = ("PureMessage::Install");
our @EXPORT_OK = qw(all_roles role_help_text);

sub new {
    my $class = shift;
    my %opts = @_;
    
    my $roles = delete $opts{roles};
    my $no_defaults = delete $opts{no_defaults};
    my $csm_opts = delete $opts{csm_opts};
    my $self = $class->SUPER::new(%opts);
    bless $self, $class;
	
    # Add default packages
    my %pkg;
	unless(@$roles == 1 and @$roles[0] eq "historiandb"){	#XXX
    	# Add default packages for roles other than Sender History Database
    	if (ref($self->{iq_conf}{role}{default}{packages}) eq "ARRAY" and
        	!$no_defaults ) {
        	my @pkgs = @{$self->{iq_conf}{role}{default}{packages}};
        	inst_log("Default packages to install: ". join(", ", @pkgs));
        	foreach my $p (@pkgs) {
            	next if $pkg{$p};
            	$pkg{$p}++;
            	push @{$self->{pkgs}}, $p;
        	}
		}
	
		if (ref($self->{iq_conf}{role}{default}{lic_packages}) eq "ARRAY" and
			!$no_defaults ) {
			my @pkgs = @{$self->{iq_conf}{role}{default}{lic_packages}};
			inst_log("Default lic_packages to install: ". join(", ", @pkgs));
			foreach my $p (@pkgs) {
				next if $pkg{$p};
				$pkg{$p}++;
				push @{$self->{lic_pkgs}}, $p;
			}
		}
	}else{
		# Packages for Standalone Sender History Database Role
		my @pkgs = qw(PureMessage-Perl ActiveState-Config ActiveState-Utils 
			   	PureMessage-System PureMessage-Config PureMessage-Utils PureMessage-Scheduler);
		if(not is_installed("PureMessage"))
		{
			push @pkgs, "PureMessage";
		}
		inst_log("Default Packages for Sender History Databse Role to work: ". join(", ", @pkgs));
		foreach my $p (@pkgs) {
			next if $pkg{$p};
			$pkg{$p}++;
			push @{$self->{pkgs}}, $p;
		}	
	}
    # Setup the selected roles
    foreach my $r (@$roles) {
        my $ref = $self->{iq_conf}{role}{$r};
        die "$r is not a valid role" unless defined $ref;

        if (ref($ref->{packages}) eq "ARRAY") {
            my @pkgs = @{$ref->{packages}};
            inst_log("Role $r packages to install: ". join(", ", @pkgs));
            foreach my $p (@pkgs) {
                next if $pkg{$p};
                $pkg{$p}++;
                push @{$self->{pkgs}}, $p;
            }
        }        
        
        if (ref($ref->{lic_packages}) eq "ARRAY") {
            my @pkgs = @{$ref->{lic_packages}};
            inst_log("Role $r licensed packages to install: ". join(", ", @pkgs));
            foreach my $p (@pkgs) {
                next if $pkg{$p};
                $pkg{$p}++;
                push @{$self->{lic_pkgs}}, $p;
            }
        }

        if (ref($ref->{rios}) eq "ARRAY") {
            foreach my $o (@{$ref->{rios}}) {
                if ($o =~ /^(\w+)(?:\.(\w+))?\s*=\s*(.+)$/) {
                    $self->{role_opts}{$1}{$2} = $3;
                }
                else {
                    inst_log("Couldn't decipher IQ role opt: $o");
                }
            }
        }
    }

    inst_log("Custom install: roles=" . join ", ", @$roles);
    $self->{roles} = $roles;

    if ($csm_opts->{host}) {
        inst_log("Connecting to $csm_opts->{host} to retrieve default install options.");
        my $opts;
        eval {
            $opts = get_install_defaults(%$csm_opts, roles => $roles);
        };
        if ($@) {
            inst_log("Error retrieving default install options: $@");
        }
        else {
            foreach my $o (keys %$opts) {
                foreach my $k (keys %{$opts->{$o}}) {
                    my $v = $opts->{$o}{$k};
                    inst_log("Default inst_opt: $o.$k = $v");
                    $self->{csm_opts}{$o}{$k} = $v;
                }
            }
        }
    }

    return $self;
}

# Download the IQ file and get the role list
sub all_roles {
    my $repo = shift;
    my $iq_conf = get_iq_conf($repo);
    die "Can't determine available roles - IQ file is not available"
        unless defined $iq_conf;

    my %roles;
    foreach my $r (keys %{$iq_conf->{role}}) {
        next if $r eq "default";
        next unless ref($iq_conf->{role}{$r}) eq "HASH";
        $roles{$r} = $iq_conf->{role}{$r}{title};
    }
    return %roles;
}

sub role_help_text {
    my $repo = shift;
    my $iq_conf = get_iq_conf($repo);
    return $iq_conf->{role}{desc};
}

1;

__END__

=head1 NAME

PureMessage::Install::Custom

=head1 SYNOPSIS

  use PureMessage::Install::Custom;
  my @roles = PureMessage::Install::Custom::roles();
  my $inst = PureMessage::Install::Custom->new( roles => [ $roles[0],
                                                           $roles[3],
                                                         ],
                                                no_defaults => 0,
                                              );

=head1 DESCRIPTION

Perform a Custom installation that installs various PureMessage roles.

=head2 new()

The constructor accepts a "roles" option, which should be an arrayref
containing the names of the roles to be installed.  The names should 
be the same as those returned by the roles() method.

The available roles are:

=over 4

=item B<Edge Server>

=item B<MTA>

=item B<CSM Server>

=item B<Enduser Web Interface>

=item B<PureMessage Database>

=back

Custom install options:

=over 4

=item no_defaults

This option can be used to avoid including the packages 
default to all roles.  The default is false (include defaults).

=item csm

You can specify the address of a CSM server to retrieve default install 
options from.

=back

=head2 all_roles()

This function returns a list of the available roles that can be installed.

=head2 role_help_text()

Return the help text describing each role.

=head1 CONFIGURATION

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2004-2017 Sophos PLC
All Rights Reserved.

=head1 SEE ALSO

pmx-setup-v6


