package PureMessage::Install::OptionValidator;
use strict;
use Exporter;
our @ISA = ('Exporter');
our @EXPORT = qw(valid);
our @EXPORT_OK = qw(valid_email valid_domain valid_smtp valid_ips valid_user valid_host valid_host_port valid_ip_cidr);
use Sophos::IP::Utils qw(:parse);
use Carp qw(croak);

sub valid {
    my ($type, $value, $protocol, $no_sbrk) = @_;
    my $sub = { email       =>  \&valid_email,
                domain      =>  \&valid_domain,
                smtp        =>  \&valid_smtp,
                ips         =>  \&valid_ips,
                user        =>  \&valid_user,
                host        =>  \&valid_host,
                host_port   =>  \&valid_host_port,
                ip_cidr     =>  \&valid_ip_cidr,
              }->{$type};
    croak "$type does not have a validator!" unless ($sub);
    return $sub->($value, $protocol, $no_sbrk);
}

my $RE_hostname_part = qr/[0-9a-zA-Z][\w-]*/;
my $RE_hostname = qr/$RE_hostname_part(?:\.$RE_hostname_part)+/;
my $RE_ip_addr = qr/\d{1,3}(?:\.\d{1,3}){3}/;

sub valid_host {
    my ($host, $protocol, $no_sbrk) = @_;
    $no_sbrk ||= 0;
    my %opts = (no_sbrk => $no_sbrk);
    return $host if $host eq '' || $host =~ /\$/;

    my ($family, $sanitized_host, $port);
    if ($host !~ /^\[$RE_ip_addr\](:\d+)?$/ &&
            $host =~ /^\[($RE_hostname|$RE_hostname_part)\](:\d+)?$/) {

        $host = sprintf "%s%s", $1, ($2 ? $2 : ':0');
        ($family, $sanitized_host, $port) = parse_hostport($host, %opts);
        if (defined $family) {
            $sanitized_host = "[$sanitized_host]";
        }
    }
    else {
        ($family, $sanitized_host, $port) = parse_hostport($host, %opts);
        if (!defined $family) {
            ($family, $sanitized_host, $port) = parse_hostport("$host:0", %opts);
        }
    }

    if (!defined $family) {
        croak "Invalid host/port\n";
    } else {
        if ($protocol =~ /^IPV4$/i and $family eq 'inet6') {
            croak "Mismatched protocol\n";
        }
        return $port ne '0' ? "${sanitized_host}:$port" : $sanitized_host;
    }
}

sub valid_host_port {
    my ($host_port, $protocol, $no_sbrk) = @_;
    $no_sbrk ||= 0;
    my %opts = (no_sbrk => $no_sbrk);

    my ($family, $host, $port) =  parse_hostport($host_port, %opts);
    if (!defined $host) {
        croak "Invalid host/port\n";
    } else {
       if ($protocol =~ /^IPV4$/i and $family eq 'inet6') {
           croak "Mismatched protocol\n";
       } else {
           return $host . ":" . $port;
       }
   }
}

sub valid_ip_cidr {
    my ($entries, $protocol, $no_sbrk) = @_;
    $no_sbrk ||= 0;
    my %opts = (no_sbrk => $no_sbrk);

    my @sanitized_cidrs;
    my @entries = split /(?:,\s*|\s+)/, $entries; # for networks
    foreach my $entry (@entries) {
        my ($family, $sanitized_cidr);
        if ($entry =~ /\$/) { # it is variable, no need to test
            push @sanitized_cidrs, $entry;
            next;
        }
        elsif ($entry =~ /\/\d+$/) {
            ($family, $sanitized_cidr) = parse_cidr($entry, %opts); 
        } else {
            ($family, $sanitized_cidr) = parse_ip($entry, %opts); 
        }

        if (!defined $family) {
            croak "Invalid cidr\n";
        }
        elsif ($protocol =~ /^IPV4$/i and $family eq 'inet6') {
            croak "Mismatched protocol\n";
        }
        push @sanitized_cidrs, $sanitized_cidr;
    }
    return join ', ', @sanitized_cidrs;
}

sub valid_email {
    my $email = shift;
    return $email if ($email =~ /^[^\s@]+\@(?:$RE_hostname|$RE_hostname_part)$/);
    croak "$email is not a valid email address\n";
}

sub valid_domain {
    my $domain = shift;
    return $domain if $domain =~ /^$RE_hostname$/;
    croak "$domain is not a valid and fully qualified hostname\n";
}

sub valid_smtp {
    my $smtp = shift;
    $smtp =~ s/^smtp://;

    my ($family, $host, $port) =  parse_hostport($smtp);
    
    if (defined $family) {
        $smtp = defined $port ? "smtp:$host:$port" : "smtp:$host:25";
    }
    else {
        croak "$smtp is not a valid mail server\n";
    }
    return $smtp;
}

sub valid_ips {
    my ($ips, $protocol, $no_sbrk) = @_;
    $no_sbrk ||= 0;
    my %opts = (no_sbrk => $no_sbrk);

    my @sanitized_ips;
    foreach my $ip (split ' ', $ips) {
        my ($family, $sanitized_ip);
        if ($ip =~ /\/\d+$/) {
            ($family, $sanitized_ip) = parse_cidr($ip, %opts);
        } else {
            ($family, $sanitized_ip) = parse_ip($ip, %opts);
        }
        if (!$family) {
            croak "Invalid IP address: $ip\n";
        }
        if (defined $protocol) {
            if ($protocol =~ /^IPV4$/i and $family eq 'inet6') {
                croak "Mismatched protocol\n";
            }
        }
        push @sanitized_ips, $sanitized_ip;
    }
    return join ' ', @sanitized_ips;
}

sub valid_user {
    my $user = shift;

    unless ($user =~ /^[a-z][a-z0-9]{0,7}$/) {
        croak "The user name '$user' is not legal.  It can only contain "
            . "lowercase letters or numbers, and cannot be more than "
            . "eight characters long.\n";
    }
    if ($user =~ /^root|bin|adm|lp|sync|daemon|mail|news|ftp$/) {
        croak "The user name '$user' is reserved.\n";
    }
    return $user;
}

1;

__END__

=head1 NAME

PureMessage::Install::OptionValidator

=head1 SYNOPSIS

  use PureMessage::Install::OptionValidator;

  # check for a valid email address
  unless (valid('email', $email)) {
      die "Bad email address: $!";
  }

=head1 DESCRIPTION

This package stores commonly used validation routines for various PureMessage
configuration options.

=head2 valid( $type, $value )

Determine the validity of the given value.  The validated value is returned on
success, and an error is raised on failure.

$type can be one of the following:

=over 4

=item email

$value must be a valid email address.

=item domain

$value must be a valid and fully qualified hostname.

=item smtp_server

$value must be a valid smtp server.  The server is not contacted.

=item ips

$value must be a valid list of space seperated IP addresses.

=back

=head1 CONFIGURATION

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2004, Sophos PLC
All Rights Reserved.
