package PureMessage::Install::Platform;

use strict;
use warnings;
use PureMessage::_sys;
use Exporter;
our @ISA = ('Exporter');
our @EXPORT_OK = qw(override supported $platform glibc_info need_to_setup_logrotate);

our $platform = "";

my %settings;
my $OVERRIDE;

sub override {
    $OVERRIDE = 1;
    %settings = @_;
}

sub glibc_info {
    $settings{os} ||= $^O;

    if ($settings{os} ne 'linux'){
        return {
            version     => undef,
            supported   => 1,
        };
    }

    my $glibc_version = PureMessage::_sys::get_glibc_version(); 
    if (!$glibc_version || $glibc_version !~ /^\d+\.\d+/) {
        return {
            defined $glibc_version ? (version => $glibc_version) : (version => "unknown"),
            supported   => 0,
        };
    }

    my ($major, $minor) = $glibc_version =~ m/^(\d+)\.(\d+).*$/;
    return { 
        version     => $glibc_version, 
        ($major == 2 && $minor >= 5) ? (supported =>  1 ) : (supported => 0)
    };
}

sub supported {
    my $supported = 0;

    $settings{os} ||= $^O;
    chomp($settings{uname_m} ||= `uname -m` || "unknown");
    chomp($settings{uname_r} ||= `uname -r` || "unknown");
    chomp($settings{uname_v} ||= `uname -v` || "unknown");
    chomp($settings{uname_s} ||= `uname -s` || "unknown");

    if ($settings{os} eq "linux") {
	    find_distro() unless $OVERRIDE;
	    if (is_x86() || is_x86_64()) {
	        $supported = supported_linux();
	    } else {
	        $platform = "$settings{os} on $settings{uname_m}";
	    }
    }
    elsif ($settings{os} eq "solaris") {
	    $supported = supported_solaris();
    }
    else {
	    $platform = "Unsupported operating system: $settings{os}";
    }
    
    $platform ||= "This unknown version of $^O";

    return $supported;
}

## OS Subroutines ##

sub supported_linux {
    my $supported = 0;

    # Redhat
    if ($settings{redhat}) {
        if (($settings{redhat} =~ /^5(?:ES|AS)/
          || $settings{redhat} =~ /^5Server 5\.[3-9]/
          || $settings{redhat} =~ /^5 .*centos/)
          || (($settings{redhat} =~ /^6 .*centos/ || $settings{redhat} =~ /^7 .*centos/ ||
               $settings{redhat} =~ /^6Server/    || $settings{redhat} =~ /^7\..*el7/) 
             && $settings{uname_m} eq 'x86_64'))
        {
            $supported = 1;
        }
        $platform = $settings{redhat_friendly};
    }

    # Debian
    if ($settings{debian}) {
        # Sophos supports debian 5, 6 and 7
        if ($settings{debian} =~ /^(\d+)/) {
            if ((($1 == 6 || $1 == 7) && $settings{uname_m} eq 'x86_64')
              ||($1 == 5)) {
                $supported = 1;
            }
        }
        $platform = "Debian Linux version $settings{debian}";
    }

    # SUSE
    if (exists $settings{suse_friendly} and 
        $settings{suse_friendly} =~ /Enterprise Server/) {
        # Sophos supports SuSE Enterprise Server 10 and 11
        # XXX: 
	    if ($settings{suse} eq '11' || $settings{suse} eq '12') {
	        $supported = 1;
	    }
	    $platform = $settings{suse_friendly};
    }

    # Ubuntu
    if (exists $settings{ubuntu}) {
        $supported = 1;
        $platform = "Ubuntu Linux version $settings{ubuntu}";
    }

    # Make sure we print a valid error message if we can't find out
    # the distribution
    $platform ||= "Unknown distribution of Linux";

    return $supported;
}

sub supported_solaris {
    my $supported = 0;

    # In order to ensure that this is a supported version of Solaris,
    # we need to make sure that the SunOS version is 5.x (meaning that
    # the Solaris version is 2.x
    my ($sunos) = $settings{uname_r} =~ /^(\d)\./;
    return 0 unless ($settings{uname_s} eq "SunOS" and $sunos);

    # Check whether the machine hardware name is in the sun4 class
    if ($settings{uname_m} =~ /^sun4.$/) {
        if ($sunos == 5) { # Solaris 2.x
            # convert from a SunOS version to a solaris version
            $settings{uname_r} =~ s/[^.]*\.//; # Strip off the 5
            $platform = "Solaris $settings{uname_r}";
            
            my $sparc_distro = find_distro();
            if ($settings{sparc} =~ /s(\d+)s_u(\d+)wos/) {
                # Sophos supports Solaris 10 update 6 and up
                if ($1 == 10 and $2 > 5) {
                    $supported = 1;
                }
            }
        }
        else {
            $platform = "SunOS$sunos";
        }
    }
    else {
        # this is unsupported machine hardware
	    $platform = "Solaris on $settings{uname_m}";
	    return 0;
    }
    
    return $supported;
}

####

sub is_x86 {
    return $settings{uname_m} =~ /^i\d86$/;
}

sub is_x86_64 {
    return $settings{uname_m} =~ /^x86_64$/;
}

sub find_distro {
    # version info files for supported linux distributions:
    my $rh_cnf = "/etc/redhat-release";
    my $suse_cnf = "/etc/SuSE-release";
    my $deb_cnf = "/etc/debian_version";
    # SuSE Enterprise 8 doesn't have a SuSE-release file, instead
    # it has a UnitedLinux-release file
    my $united_cnf = "/etc/UnitedLinux-release";
    my $sparc_cnf = "/etc/release";

    my $fh;
    if (-e $rh_cnf and not exists $settings{redhat}) {
	$settings{redhat} = `rpm -qf --qf "%{version} %{release}" $rh_cnf`;
	chomp($settings{redhat});
	unless (open($fh, "<$rh_cnf")) {
            warn "Can't open $rh_cnf\n";
            return 0;
        }
	chomp($settings{redhat_friendly} = <$fh>);
	close $fh;
        unless ($settings{redhat_friendly}) {
            warn "$rh_cnf is empty\n";
            return 0;
        }
    }
    if (-e $suse_cnf and not exists $settings{suse} ) {
	unless (open($fh, "<$suse_cnf")) {
            warn "Can't open $suse_cnf\n";
            return 0;
        }
	while (<$fh>) {
            chomp($settings{suse_friendly} ||= $_);
	    if (/VERSION = (.*)/) {
		chomp($settings{suse} = $1);
		last;
	    }
	}
	close $fh;
    }
    if (-e $deb_cnf and not (exists $settings{debian} or exists $settings{ubuntu})) {
        if ($settings{uname_v} =~ /Ubuntu/) {
            my $version = `lsb_release -r -s 2>/dev/null` || 'unknown';
            chomp($settings{ubuntu} = $version);
        } else {
            unless (open($fh, "<$deb_cnf")) {
                warn "Can't open $deb_cnf\n";
                return 0;
            }
            chomp($settings{debian} = <$fh>);
            close $fh;
        }
    }
    if (-e $united_cnf and not exists $settings{united}) {
        $settings{os} = "UnitedLinux";
	unless (open($fh, "<$united_cnf")) {
            warn "Can't open $united_cnf\n";
            return 0;
        }
	while (<$fh>) {
	    if (/VERSION = (.*)/) {
		chomp($settings{united} = $1);
		last;
	    }
	}
	close $fh;
    }

    if (-e $sparc_cnf and not exists $settings{sparc}) {
        my $os_info = `grep 'Solaris 10.*wos' $sparc_cnf`;
        my @info = split /\s+/, $os_info;
        my ($sparc) = grep { $_ =~ /wos/ } @info;
        $settings{sparc} = $sparc ? $sparc : "" ;
    }
}

sub need_to_setup_logrotate {
    if (!$platform) {
        return 0 if (!supported());
    }
    if ($platform &&
        ($platform =~ /Debian Linux version\ (\d+).*/ && $1 >= 7) ||
        ($platform =~ /Ubuntu Linux version\ (\d+).*/ && $1 >= 14)) {
        return 1;
    }
    return 0;
}

1;

__END__

=head1 NAME

PureMessage::Install::Platform

=head1 SYNOPSIS

  use PureMessage::Install::Platform qw( &supported $platform);

  # check for a supported platform
  my $supported = supported();
  
  if ($supported) {
     print "$platform is a supported platform\n";
  }
  else {
     print "$platform is not a supported platform\n";
  }

=head1 DESCRIPTION

This package is used to determine whether the platform being used is supported
in the current release of Sophos PureMessage.

=head2 supported()

This subroutine returns either 1 or 0 depending on whether the current platform 
is supported by Sophos PureMessage. If the platform cannot be determined,
either due to missing binaries or an unreadable version file, a warning will be
emitted and supported() will return 0.

supported() also sets the $platform variable to be a string version of the
platform. When the platform cannot be determined, $platform is set to $^O as a
last resort.

=head1 CONFIGURATION

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2005-2015 Sophos Group. All Rights Reserved.
