package PureMessage::Install::Utils;
use strict;
use Exporter;
use Carp qw(croak);
use ActiveState::Handy qw(run);
use ActiveState::Unix::Pw qw(su);
use ActiveState::Config::Simple qw(config_parse);
use PureMessage::Install::Prefix qw($PREFIX);
use PureMessage::Install::Log;
use PureMessage::Install::Platform qw(need_to_setup_logrotate);
use PureMessage::ServerGroup::Discover qw(scan_network);
use LWP::UserAgent;
use HTTP::Request;
use HTML::Form;
use File::Copy qw(copy);
use File::Path qw(rmtree mkpath);
use PPM::PureMessage::Utils qw( create_tmp_dir pmx_user);

our @EXPORTED_ENV = qw( DEBUG_VALIDATED_FETCH );

our @ISA = ('Exporter');
our @EXPORT_OK = qw( run_as_pmx run_as_root passwd is_installed get_iq_conf 
                    get_install_defaults discover_csms login_to_csm
                    pmx_running pmx_stop pmx_can_run extract_package_validator
                    tmp_dir_location change_logrotate_settings);

sub run_as_pmx {
    my %args = @_;

    my ($prefix, $cmd) = delete @args{qw(prefix command)};
    die "prefix is mandatory" unless defined $prefix;
    die "command is mandatory" unless defined $cmd;
    $args{login} = 1 unless exists $args{login}; # default to login shell

    my @pmx = pmx_user($prefix);
    die "No pmx user exists for prefix ($prefix)" unless @pmx;

    my %runopts;
    if ($> == $pmx[2]) {
        return $cmd if $args{_norun};
        return run("\@$cmd");
    }
    elsif ($> == 0) {
        $cmd = _pmx_cmd_env($prefix, $cmd);
        inst_log("run_as_pmx: $cmd");
        return su(%args, command => $cmd, user => $pmx[0]);
    }
    else {
        my $me = getpwuid($>);
        die "Currently $me user, can't install as pmx user ($pmx[0])";
    }
}

sub run_as_root {
    my %args = @_;

    my ($prefix, $cmd) = delete @args{qw(prefix command)};
    die "prefix is mandatory" unless defined $prefix;
    die "command is mandatory" unless defined $cmd;
    die "can't run as root unless you are root himself" if ($> != 0);

    $cmd = _pmx_cmd_env($prefix, $cmd);
    inst_log("run_as_root: $cmd");
    return $cmd if $args{_norun};
    return run("\@$cmd");
}

sub _pmx_cmd_env {
    my ($prefix,$command) = @_;

    # aggregate all the env we want to propogate to the sub processes
    my $extra = '';
    $extra .= "$_=$ENV{$_} " foreach grep { $ENV{$_} } @EXPORTED_ENV;
    my $pmx_env = "$prefix/bin/pmx-env";

    # use pmx-env if it is available
    if (-x $pmx_env) {
        return "$pmx_env $command";
    } else {
        # stick our env stuff on it
        return "$extra $command";
    }
}

sub pmx_can_run {
    my ($file) = @_;
    my ($oldeuid, $oldegid) = ($>, $));
    my ($pmxuid, $pmxgid) = (pmx_user)[2,3];
    inst_log("Checking if pmx can execute $file: "
             . "euid=$oldeuid egid=$oldegid => pmxuid=$pmxuid pmxgid=$pmxgid");
    unless(defined $oldeuid and defined $oldegid and 
           defined $pmxuid and defined $pmxgid) {
        inst_log("Can't determine all [ug]ids - returning");
        return 0;
    }
    ($>, $)) = ($pmxuid, $pmxgid);
    my $can_run = -x $file;
    ($>, $)) = ($oldeuid, $oldegid);
    inst_log("pmx user can" . ($can_run ? "" : "not") . " run $file");
    return $can_run;
}

sub is_installed {
    my $comp = shift;
    return -f "$PREFIX/etc/.packages/$comp";
}

sub passwd {
    my %opt = @_;
    my @cmd = ();

    require Expect;
    # passwd notes (/usr/bin/passwd)
    # Linux     New UNIX password:      Retype new UNIX password:
    # Solaris   New password:           Re-enter new password:
    # FreeBSD   New password:           Retype new password:
    # HP-UX     New password:           Re-enter new password:
    # AIX       lukec's New password:   Enter the new password again:

    my $user = delete $opt{user} || die "user option is mandatory for passwd";
    my $pass = delete $opt{password} || 
        die "password option is mandatory for passwd";
    chomp $pass;
    $Expect::Log_Stdout = 0;
    my $e = Expect->spawn("/usr/bin/passwd $user");
    $e->expect(20, "assword:");
    die "Didn't receive password prompt: ".$e->exp_before() if $e->exp_error();
    $e->raw_pty(1);
    sleep(1); # need to sleep to avoid deadlock with passwd
    print $e "$pass\n";
    $e->expect(20, '-re', "assword( again)?:");
    die "Didn't receive 2nd password prompt: ".$e->exp_error() if $e->exp_error();
    $e->raw_pty(1);
    sleep(1); # need to sleep to avoid deadlock with passwd
    print $e "$pass\n";
    $e->expect(5, undef); # wait for EOF for 5 secs
    $e->hard_close;
}

sub pmx_running {
    my $cmd;
    eval {
        $cmd = run_as_pmx( _norun => 1, prefix => $PREFIX,
                           command => "$PREFIX/bin/pmx status 2>&1" );
    };
    if ($@) {
        inst_log("Error running `pmx status`: $@");
        return 1; # better to error on the side of it running
    }

    inst_log("pmx_running? $cmd");
    my $status = `$cmd`;
    my $rc = $? >> 8;
    if (!$?) {
        inst_log("PMX is running: $status");
        return 1;
    }
    if ($rc != 2) {
        inst_log("pmx status reported an error [$rc]: $status");
        return 1;
    }
    inst_log("PMX is stopped: $status");
    return 0;
}

sub pmx_stop {
    my $inst_log = inst_log;

    # 'pmx stop' won't stop sendmail and postfix.
    # try to stop them here, but don't worry if we can't
    eval {
        run_as_pmx( prefix => $PREFIX,
                    command => 
                    "$PREFIX/bin/pmx-service stop smtp >>$inst_log 2>&1",
                  );
    };

    eval {
        run_as_pmx( prefix => $PREFIX,
                    command => 
                    "$PREFIX/bin/pmx-service stop postfix >>$inst_log 2>&1",
                  );
    };
    undef $@;

    # stop the rest
    eval {
        run_as_pmx( prefix => $PREFIX,
                    command => "$PREFIX/bin/pmx stop >>$inst_log 2>&1",
                  );
    };
    if ($@) {
        inst_log("Error while stopping running services: $@");
    }
}

my $iq_conf;
sub get_iq_conf {
    my $repo = shift;
    my $force = shift || undef;

    $iq_conf = undef if(defined $force);

    unless(defined $iq_conf) {
        my $ua = LWP::UserAgent->new();
        $ua->env_proxy();
        my $url = "${repo}IQ.en";
        inst_log("Retrieving IQ file from $url");
        my $req = HTTP::Request->new('GET', $url);
        my $resp = $ua->request($req);
        unless ($resp and $resp->is_success) {
            my $msg = "Cannot retrieve IQ file: $url" . 
                ($resp ? ': ' . $resp->status_line : '');
            inst_log($msg);
            die "$msg\n";
        }
        $iq_conf = $resp->content;
    }
    return config_parse($iq_conf);
}

my @csms;
sub discover_csms {
    my %opts = @_;
    my $force = delete $opts{force};

    return @csms if @csms and !$force;
    @csms = ();

    # we nee to create our own discovery packet decoder because 
    # PM::Manager::HostDiscovery can't be perlapp'd
    my $decode = sub {
        my $response = shift;
        my ($magic,$ver,$host,$port,$use_ssl,$has_defaults) = split(/:/,$response);
        return (version => $ver,
                host => $host,
                port => $port,
                use_ssl => $use_ssl,
                has_defaults => $has_defaults );
    };
    my $servers = scan_network(decoders => [ $decode ],
                               time_limit => 2,
                              );
    foreach my $k (keys %$servers) {
        my $s = $servers->{$k};
        my $host = "$servers->{$k}{proto}://$servers->{$k}{host}:$servers->{$k}{port}";
        inst_log("Found manager: $host"
                 . ($s->{has_defaults} ? " - has_defaults" : ""));
        push @csms, $host if $s->{has_defaults};
    }
    return @csms;
}

sub login_to_csm {
    my %opts = @_;
    die "host is mandatory" unless $opts{host};
    die "username is mandatory" unless $opts{username};
    die "password is mandatory" unless $opts{password};

    my $url = $opts{host};
    inst_log("Logging in to $url");
    
    my $ua = LWP::UserAgent->new();
    $ua->env_proxy();
    $ua->cookie_jar( {} );

    my $res = $ua->request( HTTP::Request->new(GET => $url) );
    my $form = (HTML::Form->parse($res->content, $res->base))[0];
    return unless defined $form;
    $form->value(user => $opts{username});
    $form->value(pass => $opts{password});
    $res = $ua->request($form->click);

    inst_log("Response from $url is " . $res->code);
    return $ua if $res->code == 302;
    return;
}

sub get_install_defaults {
    my %opts = @_;
    my ($host, $roles) = @opts{qw(host roles)};
    die "host is not defined!" unless defined $host;
    my $ua = login_to_csm(map {$_ => $opts{$_}} qw(host username password));

    my $url = "$host/install-defaults.cgi";
    if (@$roles) {
        $url .= "?role=" . join(",", @$roles);
    }
    inst_log("Retrieving install defaults file from $url");
    my $req = HTTP::Request->new('GET', $url);
    my $resp = $ua->request($req);
    unless ($resp and $resp->is_success) {
        my $msg = "Cannot retrieve install defaults file: $url" . 
            ($resp ? ': ' . $resp->status_line : '');
        inst_log($msg);
        die $msg;
    }
    my $content = $resp->content;
    inst_log("default content: $content");
    my $conf = config_parse($resp->content);
    $conf = $conf->{inst_opt} if defined $conf;
    return $conf;
}

my $VALIDATOR = undef;
sub extract_package_validator {
    my $pmx_prefix = shift;

    # If there's already a validator on the system, use it!
    my $pmx_bin_file = "$pmx_prefix/bin/pmx-validator";
    return $pmx_bin_file if -e $pmx_bin_file;

    if (defined &PerlApp::exe) {
        $VALIDATOR ||= do {
            my $validator = PerlApp::extract_bound_file("pmx-validator");
            my $umask = umask 0000;
            die "Cannot remove $pmx_bin_file - $!" 
                if (!unlink($pmx_bin_file) && -e $pmx_bin_file);

            copy($validator, $pmx_bin_file) or die "Can't copy '$validator' to '$pmx_bin_file': $!";
            chmod 0555, $pmx_bin_file;
            umask $umask;
            # unlink this file because when we update the installer itself it doesn't change its pid, which of course
            # means that PerlApp freaks out because the copy of this file it finds in its pid versioned temp directory 
            # is "Corrupt" because it no longer matches the version it has in memory.  Long story short, delete it before
            # it becomes a problem.  
            unlink $validator;
            return $pmx_bin_file;
        };
    } 
    return $VALIDATOR;
}

sub change_logrotate_settings {
    my $post = shift;

    if (need_to_setup_logrotate) {
        if ($post) {
            run('chown root:root /opt/pmx6/etc/logrotate.conf');
            run('chown -R root:root /opt/pmx6/etc/logrotate.d');
            run('chmod 644 /opt/pmx6/etc/logrotate.conf');
            run('chmod -R 644 /opt/pmx6/etc/logrotate.d/*log');
        } else {
            my $pmxuser = (pmx_user($PREFIX))[0] || 'pmx6';
            run("chown $pmxuser /opt/pmx6/etc/logrotate.conf");
            run("chown -R $pmxuser /opt/pmx6/etc/logrotate.d");
        }
    }
}

1;

__END__

=head1 NAME

PureMessage::Install::Utils

=head1 SYNOPSIS


  passwd( user => $user, pass => $pass );

=head1 DESCRIPTION

Utility functions needed for the PureMessage Install.


=head2 pmx_can_run( $file )

Determine if the specified file can be executed by the pmx user.

=head2 passwd( %opts )

Set the password of the specified user.  This should only be run by the root
user.  The following options are recognized:

=over

=item user

Which user to change the password for.

=item password

The password to be set.

=back

=head2 pmx_stop()

Stop all PureMessage services.

=head2 get_iq_conf( $repo )

Retrieve the IQ file from the given repository.

=head2 discover_csms 

Discover CSM machines on the local network.  Returns a list of hostnames
that have default insall options.

=head2 get_install_defaults( $host )

Retrieve the default install options from the specified host, and return
them as a hashref of options.

=head1 CONFIGURATION

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2004, Sophos PLC
All Rights Reserved.
